
/*  $Id: HBCIPassport.java,v 1.1 2011/05/04 22:37:43 willuhn Exp $

    This file is part of HBCI4Java
    Copyright (C) 2001-2008  Stefan Palme

    HBCI4Java is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    HBCI4Java is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.kapott.hbci.passport;

import java.util.Properties;

import org.kapott.hbci.manager.HBCIKey;
import org.kapott.hbci.structures.Konto;

/** <p>Public Interface fr HBCI-Passports. Ein HBCI-Passport ist eine Art "Ausweis",
    der individuell fr jeden Nutzer eines HBCI-Zugangs und fr jeden
    Zugangsmechanismus ist. Ein Passport reprsentiert ein HBCI-Sicherheitsmedium
    und stellt Funktionen bereit, um mit dem jeweiligen Medium zu arbeiten.
    </p><p>
    Fr jede Zugangsart gibt es eine konkrete Passport-Implementation, die dieses
    Interface implementiert. Dabei handelt es sich um</p>
    <ul>
      <li><code>HBCIPassportDDV</code> fr Zugang ber DDV mit Chipkarte</li>
      <li><code>HBCIPassportRDHNew</code> fr Zugang ber RDH mit Datei</li>
      <li><code>HBCIPassportRDH</code> fr Zugang ber RDH mit Datei (<b><em>bitte nicht mehr benutzen</em></b>;
          siehe Datei <code>README.RDHNew</code>)</li>
      <li><code>HBCIPassportPinTan</code> fr Zugang ber das PIN/TAN-Verfahren</li>
      <li><code>HBCIPassportAnonymous</code> fr den anonymen Zugang</li>
      <li><code>HBCIPassportSIZRDHFile</code> fr den Zugang ber RDH mit Datei,
          wobei als Datei eine SIZ-Schlsseldatei, wie sie z.B. von StarMoney oder GENOlite
          erzeugt wird, verwendet werden kann</li>
      <li><code>HBCIPassportRDHXFile</code> fr den Zugang ber RDH mit Datei,
          wobei als Datei eine RDH-2- oder RDH-10-Schlsseldatei verwendet wird, 
          wie sie z.B. von VR-NetWorld erzeugt wird.</li>
    </ul>
    <p>In einem Passport werden alle nutzer- und institutsspezifischen Daten verwaltet.
    Dazu gehren</p>
    <ul>
      <li>die Zugangsdaten fr den HBCI-Server der Bank (IP-Adresse, usw.)</li>
      <li>die nutzerspezifischen Zugangsdaten (Nutzerkennung, System-Kennung, usw.)</li>
      <li>die Schlsselinformationen fr die kryptografischen Operationen</li>
      <li>die gecachten BPD und die UPD</li>
    </ul>
    <p>Auerdem sind in einem Passport alle Methoden implementiert, die zur Durchfhrung
    der kryptografischen Operationen bentigt werden (verschlsseln, signieren, usw.)</p>*/
public interface HBCIPassport
{
    /** Rolle eines Passport-Objektes: Eigentmer ist Herausgeber der Nachricht.
     *  Wird in {@link org.kapott.hbci.GV.HBCIJob#addSignaturePassport(HBCIPassport, String)}
     *  bentigt. */
    public final static String ROLE_ISS="1";
    
    /** Rolle eines Passport-Objektes: Eigentmer ist Mitunterzeichner fr Nachricht.
     *  Wird in {@link org.kapott.hbci.GV.HBCIJob#addSignaturePassport(HBCIPassport, String)}
     *  bentigt. */
    public final static String ROLE_CON="3";
    
    /** Rolle eines Passport-Objektes: Eigentmer ist Zeuge oder berbringer der Nachricht.
     *  Wird in {@link org.kapott.hbci.GV.HBCIJob#addSignaturePassport(HBCIPassport, String)}
     *  bentigt. */
    public final static String ROLE_WIT="4";
    
    /** Gibt die gespeicherten BPD zurck. Die Auswertung der BPD seitens einer HBCI-Anwendung
        auf direktem Weg wird nicht empfohlen, da es keine Dokumentation ber die
        Namensgebung der einzelnen Eintrge gibt.
        @return die Bankparamterdaten oder <code>null</code>, falls diese nicht im
                Passport vorhanden sind */
    public Properties getBPD();
    
    /** Gibt die HBCI-Version zurck, die zuletzt verwendet wurde. Der hier zurckgegebene
        Wert ist der selbe, der bei der Initialisierung des 
        {@link org.kapott.hbci.manager.HBCIHandler} verwendet werden kann. Um also
        einen HBCIHandler zu erzeugen, der mit der HBCI-Version arbeitet, mit der
        ein Passport-Objekt zuletzt benutzt wurde, so kann das mit
        <code>new HBCIHandler(passport.getHBCIVersion(),passport)</code> erfolgen (vorausgesetzt,
        <code>passport.getHBCIVersion()</code> gibt einen nicht-leeren String zurck. 
        @return Die zuletzt verwendete HBCI-Version. Ist diese Information nicht 
                verfgbar, so wird ein leerer String zurckgegeben. */
    public String getHBCIVersion();
    
    /** Gibt die gespeicherten UPD (User-Parameter-Daten) zurck. Eine direkte
        Auswertung des Inhalts dieses Property-Objektes wird nicht empfohlen, da
        die Benennung der einzelnen Eintrge nicht explizit dokumentiert ist.
        @return die Userparameterdaten oder <code>null</code>, falls diese nicht im
                Passport vorhanden sind */
    public Properties getUPD();

    /** <p>Gibt die Bankleitzahl des Kreditinstitutes zurck. Bei Verwendung dieser Methode
        ist Vorsicht geboten, denn hier ist die Bankleitzahl der Bank gemeint, die
        den HBCI-Server betreibt. I.d.R. deckt sich diese BLZ zwar mit der BLZ der
        Konten des Bankkunden, es gibt aber auch Flle, wo die BLZ, die mit dieser Methode
        ermittelt wird, anders ist als die BLZ bei den Kontoverbindungen des
        Kunden.
        </p><p>
        Fr die Ermittlung der BLZ fr die Kontodaten sollte statt dessen die Methode
        {@link #getAccounts()} benutzt werden.
        </p>
        @return die BLZ der Bank */
    public String getBLZ();

    /** Gibt den Lndercode der Bank zurck. Fr deutsche Banken ist das der String
        "<code>DE</code>".
        @return Lndercode der Bank */
    public String getCountry();
    
    /** Gibt ein Array mit Kontoinformationen zurck. Auf die hier zurckgegebenen Konten kann via
        HBCI zugegriffen werden. Nicht jede Bank untersttzt diese Abfrage, so dass dieses Array
        u.U. auch leer sein kann, obwohl natrlich via HBCI auf bestimmte Konten zugegriffen werden
        kann. In diesem Fall mssen die Kontoinformationen anderweitig ermittelt werden (manuelle
        Eingabe des Anwenders).
        @return Array mit Kontoinformationen ber verfgbare HBCI-Konten */ 
    public Konto[] getAccounts();
    
    /** Ausfllen fehlender Kontoinformationen. In der Liste der verfgbaren Konten (siehe
        {@link #getAccounts()}) wird nach einem Konto gesucht, welches die
        gleiche Kontonummer hat wie das bergebene Konto <code>account</code>. Wird ein solches
        Konto gefunden, so werden die Daten dieses gefundenen Kontos in das <code>account</code>-Objekt
        bertragen.<p/>
        Diese Methode kann benutzt werden, wenn zu einem Konto nicht alle Daten bekannt sind, wenigstens
        aber die Kontonummer.
        @param account unvollstndige Konto-Informationen, bei denen die fehlenden Daten nachgetragen
               werden */
    public void fillAccountInfo(Konto account);
    
    /** Gibt ein Konto-Objekt zu einer bestimmten Kontonummer zurck. Dazu wird die Liste, die via
        {@link #getAccounts()} erzeugt wird, nach der Kontonummer durchsucht. Es wird in
        jedem Fall ein nicht-leeres Kontoobjekt zurckgegeben. Wird die Kontonummer jedoch nicht in 
        der Liste gefunden, so wird das Konto-Objekt aus den "allgemeinen" Bank-Daten gebildet:
        Kontonummer=<code>number</code>; Lnderkennung, BLZ und Kunden-ID aus dem Passport-Objekt;
        Whrung des Kontos hart auf "EUR"; Name=Kunden-ID.
        @param number die Kontonummer, fr die ein Konto-Objekt erzeugt werden soll
        @return ein Konto-Objekt, welches mindestens die Kontonummer enthlt. Wenn 
                verfgbar, so sind auch die restlichen Informationen ber dieses Konto (BLZ,
                Inhaber, Whrung usw.) ausgefllt */
    public Konto getAccount(String number);

    /** Gibt den Hostnamen des HBCI-Servers fr dieses Passport zurck. Handelt es sich bei
        dem Passport-Objekt um ein PIN/TAN-Passport, so enthlt dieser String die URL,
        die fr die HTTPS-Kommunikation mit dem HBCI-Server der Bank benutzt wird. 
        @return Hostname oder IP-Adresse des HBCI-Servers */
    public String getHost();
    
    /** Gibt die TCP-Portnummer auf dem HBCI-Server zurck, zu der eine
        HBCI-Verbindung aufgebaut werden soll. In der Regel ist das der Port 3000,
        fr PIN/TAN-Passports wird hier 443 (fr HTTPS-Port) zurckgegeben.
        Der zu benutzende TCP-Port fr die Kommunikation kannn mit 
        {@link #setPort(Integer)} gendert werden.
        @return TCP-Portnummer auf dem HBCI-Server */
    public Integer getPort();
    
    /** Gibt zurck, welcher Datenfilter fr die Kommunikation mit dem HBCI-Server
        verwendet wird. Gltige Bezeichner fr Filter sind "<code>None</code>" und
        "<code>Base64</code>". */
    public String getFilterType();
    
    /** Gibt die Benutzerkennung zurck, die zur Authentifikation am
        HBCI-Server benutzt wird.
        @return Benutzerkennung fr Authentifikation */
    public String getUserId();
    public String getCustomerId(int idx);
    
    /** <p>Gibt die Kunden-ID zurck, die von <em>HBCI4Java</em> fr die
        Initialisierung eines Dialoges benutzt wird. Zu einer Benutzerkennung
        ({@link #getUserId()}), welche jeweils an ein bestimmtes Medium
        gebunden ist, kann es mehrere Kunden-IDs geben. Die verschiedenen
        Kunden-IDs entsprechen verschiedenen Rollen, in denen der Benutzer
        auftreten kann.</p>
        <p>In den meisten Fllen gibt es zu einer Benutzerkennung nur eine
        einzige Kunden-ID. Wird von der Bank keine Kunden-ID explizit vergeben,
        so ist die Kunden-ID identisch mit der Benutzerkennung.</p>
        <p>Siehe dazu auch 
        {@link org.kapott.hbci.GV.HBCIJob#addToQueue(String)}.
        </p>
        @return Kunden-ID fr die HBCI-Kommunikation */
    public String getCustomerId();
    public boolean isSupported();
    
    public boolean needInstKeys();
    public boolean needUserKeys();
    
    public boolean hasInstSigKey();
    public boolean hasInstEncKey();
    
    public boolean hasMySigKey();
    public boolean hasMyEncKey();
    
    public void clearInstSigKey();
    public void clearInstEncKey();

    public HBCIKey getMyPublicSigKey();
    public HBCIKey getMyPublicEncKey();
    public HBCIKey getMyPublicDigKey();
    public HBCIKey getMyPrivateSigKey();
    public HBCIKey getMyPrivateEncKey();
    public HBCIKey getMyPrivateDigKey();
    public HBCIKey getInstSigKey();
    public HBCIKey getInstEncKey();

    /** Gibt die Versionsnummer der lokal gespeicherten BPD zurck. Sind keine
        BPD vorhanden, so wird "0" zurckgegeben. Leider benutzen einige Banken
        "0" auch als Versionsnummer fr die tatschlich vorhandenen BPD, so 
        dass bei diesen Banken auch dann "0" zurckgegeben wird, wenn in Wirklichkeit
        BPD vorhanden sind.
        @return Versionsnummer der lokalen BPD */
    public String getBPDVersion();

    /** Gibt die Versionsnummer der lokal gespeicherten UPD zurck. Sind keine UPD
        lokal vorhanden, so wird "0" zurckgegeben. Siehe dazu auch
        {@link #getBPDVersion()}.
        @return Versionsnummer der lokalen UPD */
    public String getUPDVersion();

    /** Gibt den Namen des Kreditinstitutes zurck. Diese Information wird aus
        den BPD ermittelt. Sind keine BPD vorhanden bzw. steht da kein Name drin,
        so wird <code>null</code> zurckgegeben.
        @return Name des Kreditinstitutes */
    public String getInstName();
    public int getMaxGVperMsg();
    public int getMaxMsgSizeKB();

    /** Gibt eine Liste aller untersttzten Sprachcodes zurck. Die einzelnen
        Codes stehen dabei fr folgende Sprachen:
        <ul>
          <li>1 - deutsch</li>
          <li>2 - englisch</li>
          <li>3 - franzsisch</li>
        </ul>
        @return Liste aller untersttzten Sprachen (1,2,3) */
    public String[] getSuppLangs();

    /** <p>Gibt eine Liste aller untersttzten HBCI-Versionen zurck.
        Die einzelnen Strings fr die Versionen sind die gleichen, wie sie in der Methode
        {@link org.kapott.hbci.manager.HBCIHandler#HBCIHandler(String,org.kapott.hbci.passport.HBCIPassport)}
         verwendet werden knnen.
        </p><p>
        Zustzlich zu den hier zurckgegebenen HBCI-Versions-Codes gibt es einige 
        spezielle Codes. Siehe dazu die Dokumentation zu
        {@link org.kapott.hbci.manager.HBCIHandler#HBCIHandler(String,org.kapott.hbci.passport.HBCIPassport)}
        </p>
        @return eine Liste aller von der Bank untersttzten HBCI-Versionen */
    public String[] getSuppVersions();

    /** Gibt die Standardsprache des HBCI-Servers zurck. Zu den Bedeutungen der
        Sprachcodes siehe {@link #getSuppLangs()}.
        @return Standardsprache (1,2 oder 3) */
    public String getDefaultLang();

    /** <p>Gibt eine Liste der vom HBCI-Server untersttzten Sicherheitsmechanismen
        zurck. Gltige Werte fr jeden einzelnen String sind <code>RDH</code> bzw.
        <code>DDV</code>.<p/><p>
        Die Untersttzung des PIN/TAN-Verfahrens kann mit dieser Methode nicht
        ermittelt werden.</p>
        @return eine Liste der untersttzten Sicherheitsmechanismen. Jeder Listeneintrag
                ist wieder ein Stringarray mit zwei Elementen: dem Namen des
                Mechanismus und der Versionsnummer dieses Mechanismus */
    public String[][] getSuppSecMethods();
    public String[][] getSuppCompMethods();
    
    /** Lschen der lokal gespeicherten BPD. Damit kann erzwungen werden, dass
        die BPD beim nchsten HBCI-Dialog erneut abgeholt werden. */
    public void clearBPD();
    /** Lschen der lokal gespeicherten UPD. Damit kann erzwungen werden, dass
        die UPD beim nchsten HBCI-Dialog erneut abgeholt werden. */
    public void clearUPD();

    public void setCountry(String country);
    public void setBLZ(String blz);

    /** <p>Manuelles Setzen der Adresse des HBCI-Servers. Das kann evtl. ntig
        sein, wenn sich die Zugangsdaten des Server gendert haben. Die nderungen
        werden permanent gespeichert, nachdem die neuen Werte wenigstens einmal in
        einem HBCI-Dialog benutzt wurden oder mit 
        {@link #saveChanges()} explizit gespeichert
        werden. Diese permanente Speicherung wird allerdings
        nur bei RDH- oder PIN/TAN-Passports durchgefhrt. Um die Daten bei DDV-Passports
        permanent auf der Chipkarte zu speichern, ist der HBCI-PassportEditor
        ntig</p><p>(es wre kein Problem, diese Daten sofort auf der Chipkarte zu speichern,
        allerdings besteht dann die Gefahr, dass man "aus Versehen" falsche
        Daten auf der Chipkarte ablegt und die richtigen Daten nicht wieder restaurieren
        kann, da es bei DDV-Zugngen i.d.R. keine Begleitbriefe von der Bank gibt,
        in denen die korrekten Zugangsdaten aufgelistet sind).</p>
        <p>Fr das HBCI-PIN/TAN-Verfahren wird als <code>host</code> die URL angegeben,
        welche fr die Behandlung der HBCI-PIN/TAN-Nachrichten zu benutzen ist
        (z.B. <code><em>www.meinebank.de/pintan/PinTanServlet</em></code>). Soll ein
        anderer Port als der normale HTTPS-Port 443 benutzt werden, so darf die neue
        Portnummer <em>nicht</em> in der URL kodiert werden. Statt dessen muss die
        Methode {@link #setPort(Integer)} benutzt werden, um die Portnummer zu
        setzen.</p>
        @param host die neue Adresse, unter der der HBCI-Server zu erreichen ist */
    public void setHost(String host);
    
    /** Setzen des TCP-Ports, der fr HBCI-Verbindungen benutzt wird. Bei HBCI-PIN/TAN-
     *  Passports wird der Port mit <code>443</code> vorinitialisiert, fr alle anderen
     *  "normalen" HBCI-Verbindungstypen mit <code>3000</code>. Diese Methode kann
     *  benutzt werden, wenn eine andere Portnummer als die default-Nummer benutzt
     *  werden soll. Die Portnummer fr ein Passport kann auch mit dem 
     *  <em>HBCI4Java Passport Editor</em> gendert werden.
     *  @param port neue TCP-Portnummer, die fr ausgehende Verbindungen benutzt 
     *         werden soll */
    public void setPort(Integer port);
    
    public void setFilterType(String filter);
    public void setUserId(String userid);
    
    /** Setzen der zu verwendenden Kunden-ID. Durch Aufruf dieser Methode wird die
        Kunden-ID gesetzt, die beim nchsten Ausfhren eines HBCI-Dialoges
        ({@link org.kapott.hbci.manager.HBCIHandler#execute()})
        benutzt wird. Diese neue Kunden-ID wird dann auerdem permanent im
        jeweiligen Sicherheitsmedium gespeichert (sofern das von dem Medium 
        untersttzt wird). 
        @param customerid die zu verwendende Kunden-ID; wird keine customerid
        angegeben (<code>null</code> oder ""), so wird automatisch die 
        User-ID verwendet. 
        
        @see #getCustomerId() */
    public void setCustomerId(String customerid);
    public boolean onlyBPDGVs();
    
    /** Speichern der nderungen an den Passport-Daten. Diese Methode sollte eigentlich
        niemals manuell aus einer Anwendung heraus aufgerufen werden, sondern wird 
        vom HBCI-Kernel benutzt. Das manuelle Aufrufen von <code>saveChanges</code>
        ist nur dann sinnvoll, wenn irgendwelche Passport-Daten manuell verndert
        werden ({@link #setHost(String)},
        {@link #clearBPD()} usw.) und diese nderungen
        explizit gespeichert werden sollen. */
    public void saveChanges();

    /** <p>Schlieen eines Passport-Objektes. Diese Methode wird normalerweise
        nicht manuell aufgerufen, da das bereits von 
        {@link org.kapott.hbci.manager.HBCIHandler#close()} erledigt
        wird. Wurde jedoch ein Passport-Objekt erzeugt, und das anschlieende
        Erzeugen eines HBCIHandler-Objektes schlgt fehlt, dann ist das Passport
        immer noch geffnet und sollte mit dieser Methode geschlossen werden, falls
        es nicht weiterbenutzt werden soll.</p><p>
        Am Ende eines Programmes sollte also in jedem Fall entweder ein erfolgreiches
        {@link org.kapott.hbci.manager.HBCIHandler#close()} oder
        wenigstens ein {@link org.kapott.hbci.passport.HBCIPassport#close()}
        fr jedes erzeugte Passport-Objekt stehen. Das ist vor allem fr
        Passport-Varianten wichtig, die auf einer Chipkarte basieren, da mit dieser
        Methode die entsprechenden Ressourcen wieder freigegeben werden. */
    public void close();
    
    /** Synchronisation der Signatur-ID erzwingen (nur fr RDH-Passports sinnvoll).
        Diese Methode kann
        aufgerufen werden, <em>nachdem</em> ein Passport erzeugt wurde,
        aber <em>bevor</em> damit ein neues <code>HBCIHandler</code>-Objekt
        erzeugt wird. Durch den Aufruf dieser Methode wird veranlasst, dass
        beim Erzeugen eines <code>HBCIHandler</code>-Objektes mit diesem
        Passport die Signatur-ID des Passports synchronisiert wird.*/
    public void syncSigId();

    /** Synchronisation der System-ID (nur fr RDH-Passports sinnvoll).
        Diese Methode kann
        aufgerufen werden, <em>nachdem</em> ein Passport erzeugt wurde,
        aber <em>bevor</em> damit ein neues <code>HBCIHandler</code>-Objekt
        erzeugt wird. Durch den Aufruf dieser Methode wird veranlasst, dass
        beim Erzeugen eines <code>HBCIHandler</code>-Objektes mit diesem
        Passport die System-ID des Passports synchronisiert wird. */
    public void syncSysId();
    
    /** ndern des Passwortes fr die Schlsseldatei. Der Aufruf dieser
        Methode bewirkt, dass <em>HBCI4Java</em> via Callback-Mechanismus 
        (<code>NEED_PASSPHRASE_SAVE</code>) nach dem neuen Passwort fr die 
        Schlsseldatei fragt. Anschlieend wird das Medium unter Verwendung des
        neuen Passwortes automatisch neu gespeichert. */
    public void changePassphrase();
    
    /** Speichern zustzlicher Daten im Passport-Objekt. Diese Methode ermglicht
        das Speichern zustzlicher Informationen (Objekte), die diesem Passport
        zugeordnet sind. Die Funktionsweise ist analog zur Verwendung einer
        Hashtable, es wird also ein Objekt <code>o</code> unter dem Identifikations-String
        <code>id</code> gespeichert. Mit {@link #getClientData(String)}
        kann das entsprechende Objekt wieder ausgelesen werden. Die mit dieser Methode
        gesetzten Daten werden <em>nicht</em> mit in der Schlsseldatei (Passport-Datei)
        abgelegt, d.h. die Lebensdauer dieser Daten entspricht nur der Lebensdauer
        des Passport-Objektes.
        @param id Identifikationsstring fr das zu speichernde Objekt
        @param o zu speicherndes Objekt */
    public void setClientData(String id,Object o);
    
    /** Holen von clientseitig gespeicherten zustzlichen Daten. Mit dieser Methode
        knnen die zustzlichen Daten, die via {@link #setClientData(String,Object)}
        im Passport gespeichert wurden, wieder ausgelesen werden. Auch das Objekt,
        das beim Erzeugen eines Passport-Objektes als <code>init</code>-Parameter bergeben wurde 
        (siehe {@link org.kapott.hbci.passport.AbstractHBCIPassport#getInstance(String,Object)}),
        kann damit ausgelesen werden (mit <code>id="init"</code>).
        @param id  Identifikationsstring des auszulesenden Objektes
        @return Objekt, welches mit {@link #setClientData(String,Object)}
        im Passport gespeichert wurde.*/ 
    public Object getClientData(String id);
}
